# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..types import client_list_params, client_create_params, client_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.client import Client
from ..types.client_list_response import ClientListResponse

__all__ = ["ClientsResource", "AsyncClientsResource"]


class ClientsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ClientsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/identety/identety-python-sdk#accessing-raw-response-data-eg-headers
        """
        return ClientsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ClientsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/identety/identety-python-sdk#with_streaming_response
        """
        return ClientsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        type: Literal["public", "private", "m2m"],
        allowed_grants: List[Literal["authorization_code", "client_credentials", "refresh_token"]]
        | NotGiven = NOT_GIVEN,
        allowed_scopes: List[str] | NotGiven = NOT_GIVEN,
        redirect_uris: List[str] | NotGiven = NOT_GIVEN,
        settings: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Client:
        """
        Create client

        Args:
          name: Client Name

          type: Client type

          allowed_grants: Allowed Grants

          allowed_scopes: Allowed Scopes

          redirect_uris: Redirect URIs

          settings: Client Settings

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/clients",
            body=maybe_transform(
                {
                    "name": name,
                    "type": type,
                    "allowed_grants": allowed_grants,
                    "allowed_scopes": allowed_scopes,
                    "redirect_uris": redirect_uris,
                    "settings": settings,
                },
                client_create_params.ClientCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Client,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Client:
        """
        Get client details by id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/clients/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Client,
        )

    def update(
        self,
        id: str,
        *,
        name: str,
        allowed_grants: List[Literal["authorization_code", "client_credentials", "refresh_token"]]
        | NotGiven = NOT_GIVEN,
        allowed_scopes: List[str] | NotGiven = NOT_GIVEN,
        redirect_uris: List[str] | NotGiven = NOT_GIVEN,
        settings: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Client:
        """
        Update client

        Args:
          name: Client Name

          allowed_grants: Allowed Grants

          allowed_scopes: Allowed Scopes

          redirect_uris: Redirect URIs

          settings: Client Settings

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/clients/{id}",
            body=maybe_transform(
                {
                    "name": name,
                    "allowed_grants": allowed_grants,
                    "allowed_scopes": allowed_scopes,
                    "redirect_uris": redirect_uris,
                    "settings": settings,
                },
                client_update_params.ClientUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Client,
        )

    def list(
        self,
        *,
        columns: Literal[
            "id",
            "client_id",
            "client_secret",
            "name",
            "type",
            "redirect_uris",
            "allowed_scopes",
            "allowed_grants",
            "is_active",
            "require_pkce",
            "settings",
            "tenant_id",
            "created_at",
            "updated_at",
        ],
        limit: float | NotGiven = NOT_GIVEN,
        page: float | NotGiven = NOT_GIVEN,
        sort: Literal["asc", "desc"] | NotGiven = NOT_GIVEN,
        sort_by: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ClientListResponse:
        """
        Get all clients

        Args:
          columns: Comma separated column names

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/clients",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "columns": columns,
                        "limit": limit,
                        "page": page,
                        "sort": sort,
                        "sort_by": sort_by,
                    },
                    client_list_params.ClientListParams,
                ),
            ),
            cast_to=ClientListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Client:
        """
        Delete client

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/clients/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Client,
        )


class AsyncClientsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncClientsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/identety/identety-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncClientsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncClientsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/identety/identety-python-sdk#with_streaming_response
        """
        return AsyncClientsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        type: Literal["public", "private", "m2m"],
        allowed_grants: List[Literal["authorization_code", "client_credentials", "refresh_token"]]
        | NotGiven = NOT_GIVEN,
        allowed_scopes: List[str] | NotGiven = NOT_GIVEN,
        redirect_uris: List[str] | NotGiven = NOT_GIVEN,
        settings: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Client:
        """
        Create client

        Args:
          name: Client Name

          type: Client type

          allowed_grants: Allowed Grants

          allowed_scopes: Allowed Scopes

          redirect_uris: Redirect URIs

          settings: Client Settings

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/clients",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "type": type,
                    "allowed_grants": allowed_grants,
                    "allowed_scopes": allowed_scopes,
                    "redirect_uris": redirect_uris,
                    "settings": settings,
                },
                client_create_params.ClientCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Client,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Client:
        """
        Get client details by id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/clients/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Client,
        )

    async def update(
        self,
        id: str,
        *,
        name: str,
        allowed_grants: List[Literal["authorization_code", "client_credentials", "refresh_token"]]
        | NotGiven = NOT_GIVEN,
        allowed_scopes: List[str] | NotGiven = NOT_GIVEN,
        redirect_uris: List[str] | NotGiven = NOT_GIVEN,
        settings: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Client:
        """
        Update client

        Args:
          name: Client Name

          allowed_grants: Allowed Grants

          allowed_scopes: Allowed Scopes

          redirect_uris: Redirect URIs

          settings: Client Settings

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/clients/{id}",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "allowed_grants": allowed_grants,
                    "allowed_scopes": allowed_scopes,
                    "redirect_uris": redirect_uris,
                    "settings": settings,
                },
                client_update_params.ClientUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Client,
        )

    async def list(
        self,
        *,
        columns: Literal[
            "id",
            "client_id",
            "client_secret",
            "name",
            "type",
            "redirect_uris",
            "allowed_scopes",
            "allowed_grants",
            "is_active",
            "require_pkce",
            "settings",
            "tenant_id",
            "created_at",
            "updated_at",
        ],
        limit: float | NotGiven = NOT_GIVEN,
        page: float | NotGiven = NOT_GIVEN,
        sort: Literal["asc", "desc"] | NotGiven = NOT_GIVEN,
        sort_by: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ClientListResponse:
        """
        Get all clients

        Args:
          columns: Comma separated column names

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/clients",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "columns": columns,
                        "limit": limit,
                        "page": page,
                        "sort": sort,
                        "sort_by": sort_by,
                    },
                    client_list_params.ClientListParams,
                ),
            ),
            cast_to=ClientListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Client:
        """
        Delete client

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/clients/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Client,
        )


class ClientsResourceWithRawResponse:
    def __init__(self, clients: ClientsResource) -> None:
        self._clients = clients

        self.create = to_raw_response_wrapper(
            clients.create,
        )
        self.retrieve = to_raw_response_wrapper(
            clients.retrieve,
        )
        self.update = to_raw_response_wrapper(
            clients.update,
        )
        self.list = to_raw_response_wrapper(
            clients.list,
        )
        self.delete = to_raw_response_wrapper(
            clients.delete,
        )


class AsyncClientsResourceWithRawResponse:
    def __init__(self, clients: AsyncClientsResource) -> None:
        self._clients = clients

        self.create = async_to_raw_response_wrapper(
            clients.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            clients.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            clients.update,
        )
        self.list = async_to_raw_response_wrapper(
            clients.list,
        )
        self.delete = async_to_raw_response_wrapper(
            clients.delete,
        )


class ClientsResourceWithStreamingResponse:
    def __init__(self, clients: ClientsResource) -> None:
        self._clients = clients

        self.create = to_streamed_response_wrapper(
            clients.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            clients.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            clients.update,
        )
        self.list = to_streamed_response_wrapper(
            clients.list,
        )
        self.delete = to_streamed_response_wrapper(
            clients.delete,
        )


class AsyncClientsResourceWithStreamingResponse:
    def __init__(self, clients: AsyncClientsResource) -> None:
        self._clients = clients

        self.create = async_to_streamed_response_wrapper(
            clients.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            clients.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            clients.update,
        )
        self.list = async_to_streamed_response_wrapper(
            clients.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            clients.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .client import Client
from .._models import BaseModel

__all__ = ["ClientListResponse"]


class ClientListResponse(BaseModel):
    meta: object

    nodes: List[Client]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Client"]


class Client(BaseModel):
    id: str

    allowed_grants: List[str] = FieldInfo(alias="allowedGrants")

    allowed_scopes: List[str] = FieldInfo(alias="allowedScopes")

    client_id: str = FieldInfo(alias="clientId")

    client_secret: str = FieldInfo(alias="clientSecret")

    is_active: bool = FieldInfo(alias="isActive")

    name: str

    redirect_uris: List[str] = FieldInfo(alias="redirectUris")

    settings: object

    type: Literal["public", "private", "m2m"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ClientListParams"]


class ClientListParams(TypedDict, total=False):
    columns: Required[
        Literal[
            "id",
            "client_id",
            "client_secret",
            "name",
            "type",
            "redirect_uris",
            "allowed_scopes",
            "allowed_grants",
            "is_active",
            "require_pkce",
            "settings",
            "tenant_id",
            "created_at",
            "updated_at",
        ]
    ]
    """Comma separated column names"""

    limit: float

    page: float

    sort: Literal["asc", "desc"]

    sort_by: Annotated[str, PropertyInfo(alias="sortBy")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["UserUpdateParams", "Address"]


class UserUpdateParams(TypedDict, total=False):
    address: Required[Address]

    family_name: Required[Annotated[str, PropertyInfo(alias="familyName")]]

    given_name: Required[Annotated[str, PropertyInfo(alias="givenName")]]

    locale: Required[str]

    metadata: Required[object]

    name: Required[str]

    picture: Required[str]


class Address(TypedDict, total=False):
    country: Required[str]

    locality: Required[str]

    postal_code: Required[Annotated[str, PropertyInfo(alias="postalCode")]]

    region: Required[str]

    street_address: Required[Annotated[str, PropertyInfo(alias="streetAddress")]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .app import (
    AppResource,
    AsyncAppResource,
    AppResourceWithRawResponse,
    AsyncAppResourceWithRawResponse,
    AppResourceWithStreamingResponse,
    AsyncAppResourceWithStreamingResponse,
)
from .orgs import (
    OrgsResource,
    AsyncOrgsResource,
    OrgsResourceWithRawResponse,
    AsyncOrgsResourceWithRawResponse,
    OrgsResourceWithStreamingResponse,
    AsyncOrgsResourceWithStreamingResponse,
)
from .roles import (
    RolesResource,
    AsyncRolesResource,
    RolesResourceWithRawResponse,
    AsyncRolesResourceWithRawResponse,
    RolesResourceWithStreamingResponse,
    AsyncRolesResourceWithStreamingResponse,
)
from .users import (
    UsersResource,
    AsyncUsersResource,
    UsersResourceWithRawResponse,
    AsyncUsersResourceWithRawResponse,
    UsersResourceWithStreamingResponse,
    AsyncUsersResourceWithStreamingResponse,
)
from .clients import (
    ClientsResource,
    AsyncClientsResource,
    ClientsResourceWithRawResponse,
    AsyncClientsResourceWithRawResponse,
    ClientsResourceWithStreamingResponse,
    AsyncClientsResourceWithStreamingResponse,
)

__all__ = [
    "AppResource",
    "AsyncAppResource",
    "AppResourceWithRawResponse",
    "AsyncAppResourceWithRawResponse",
    "AppResourceWithStreamingResponse",
    "AsyncAppResourceWithStreamingResponse",
    "ClientsResource",
    "AsyncClientsResource",
    "ClientsResourceWithRawResponse",
    "AsyncClientsResourceWithRawResponse",
    "ClientsResourceWithStreamingResponse",
    "AsyncClientsResourceWithStreamingResponse",
    "UsersResource",
    "AsyncUsersResource",
    "UsersResourceWithRawResponse",
    "AsyncUsersResourceWithRawResponse",
    "UsersResourceWithStreamingResponse",
    "AsyncUsersResourceWithStreamingResponse",
    "OrgsResource",
    "AsyncOrgsResource",
    "OrgsResourceWithRawResponse",
    "AsyncOrgsResourceWithRawResponse",
    "OrgsResourceWithStreamingResponse",
    "AsyncOrgsResourceWithStreamingResponse",
    "RolesResource",
    "AsyncRolesResource",
    "RolesResourceWithRawResponse",
    "AsyncRolesResourceWithRawResponse",
    "RolesResourceWithStreamingResponse",
    "AsyncRolesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.


from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["User", "Address"]


class Address(BaseModel):
    country: str

    locality: str

    postal_code: str = FieldInfo(alias="postalCode")

    region: str

    street_address: str = FieldInfo(alias="streetAddress")


class User(BaseModel):
    id: str

    address: Address

    family_name: str = FieldInfo(alias="familyName")

    given_name: str = FieldInfo(alias="givenName")

    locale: str

    metadata: object

    name: str

    picture: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ClientCreateParams"]


class ClientCreateParams(TypedDict, total=False):
    name: Required[str]
    """Client Name"""

    type: Required[Literal["public", "private", "m2m"]]
    """Client type"""

    allowed_grants: Annotated[
        List[Literal["authorization_code", "client_credentials", "refresh_token"]], PropertyInfo(alias="allowedGrants")
    ]
    """Allowed Grants"""

    allowed_scopes: Annotated[List[str], PropertyInfo(alias="allowedScopes")]
    """Allowed Scopes"""

    redirect_uris: Annotated[List[str], PropertyInfo(alias="redirectUris")]
    """Redirect URIs"""

    settings: object
    """Client Settings"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from identety import Identety, AsyncIdentety
from tests.utils import assert_matches_type
from identety.types import User, UserListResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUsers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Identety) -> None:
        user = client.users.create(
            address={
                "country": "USA",
                "locality": "New York",
                "postal_code": "10001",
                "region": "NY",
                "street_address": "123 Main St",
            },
            email="john@example.com",
            family_name="Doe",
            given_name="John",
            locale="en-US",
            metadata={"customField": "value"},
            name="John Doe",
            password="password123",
            picture="https://example.com/photo.jpg",
        )
        assert_matches_type(User, user, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Identety) -> None:
        response = client.users.with_raw_response.create(
            address={
                "country": "USA",
                "locality": "New York",
                "postal_code": "10001",
                "region": "NY",
                "street_address": "123 Main St",
            },
            email="john@example.com",
            family_name="Doe",
            given_name="John",
            locale="en-US",
            metadata={"customField": "value"},
            name="John Doe",
            password="password123",
            picture="https://example.com/photo.jpg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = response.parse()
        assert_matches_type(User, user, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Identety) -> None:
        with client.users.with_streaming_response.create(
            address={
                "country": "USA",
                "locality": "New York",
                "postal_code": "10001",
                "region": "NY",
                "street_address": "123 Main St",
            },
            email="john@example.com",
            family_name="Doe",
            given_name="John",
            locale="en-US",
            metadata={"customField": "value"},
            name="John Doe",
            password="password123",
            picture="https://example.com/photo.jpg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = response.parse()
            assert_matches_type(User, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Identety) -> None:
        user = client.users.retrieve(
            "id",
        )
        assert_matches_type(User, user, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Identety) -> None:
        response = client.users.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = response.parse()
        assert_matches_type(User, user, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Identety) -> None:
        with client.users.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = response.parse()
            assert_matches_type(User, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Identety) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.users.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: Identety) -> None:
        user = client.users.update(
            id="id",
            address={
                "country": "USA",
                "locality": "New York",
                "postal_code": "10001",
                "region": "NY",
                "street_address": "123 Main St",
            },
            family_name="Doe",
            given_name="John",
            locale="en-US",
            metadata={"customField": "value"},
            name="John Doe",
            picture="https://example.com/photo.jpg",
        )
        assert_matches_type(User, user, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Identety) -> None:
        response = client.users.with_raw_response.update(
            id="id",
            address={
                "country": "USA",
                "locality": "New York",
                "postal_code": "10001",
                "region": "NY",
                "street_address": "123 Main St",
            },
            family_name="Doe",
            given_name="John",
            locale="en-US",
            metadata={"customField": "value"},
            name="John Doe",
            picture="https://example.com/photo.jpg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = response.parse()
        assert_matches_type(User, user, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Identety) -> None:
        with client.users.with_streaming_response.update(
            id="id",
            address={
                "country": "USA",
                "locality": "New York",
                "postal_code": "10001",
                "region": "NY",
                "street_address": "123 Main St",
            },
            family_name="Doe",
            given_name="John",
            locale="en-US",
            metadata={"customField": "value"},
            name="John Doe",
            picture="https://example.com/photo.jpg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = response.parse()
            assert_matches_type(User, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Identety) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.users.with_raw_response.update(
                id="",
                address={
                    "country": "USA",
                    "locality": "New York",
                    "postal_code": "10001",
                    "region": "NY",
                    "street_address": "123 Main St",
                },
                family_name="Doe",
                given_name="John",
                locale="en-US",
                metadata={"customField": "value"},
                name="John Doe",
                picture="https://example.com/photo.jpg",
            )

    @parametrize
    def test_method_list(self, client: Identety) -> None:
        user = client.users.list(
            columns="id",
        )
        assert_matches_type(UserListResponse, user, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Identety) -> None:
        user = client.users.list(
            columns="id",
            limit=0,
            page=0,
            sort="asc",
            sort_by="sortBy",
        )
        assert_matches_type(UserListResponse, user, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Identety) -> None:
        response = client.users.with_raw_response.list(
            columns="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = response.parse()
        assert_matches_type(UserListResponse, user, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Identety) -> None:
        with client.users.with_streaming_response.list(
            columns="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = response.parse()
            assert_matches_type(UserListResponse, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Identety) -> None:
        user = client.users.delete(
            "id",
        )
        assert_matches_type(User, user, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Identety) -> None:
        response = client.users.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = response.parse()
        assert_matches_type(User, user, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Identety) -> None:
        with client.users.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = response.parse()
            assert_matches_type(User, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Identety) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.users.with_raw_response.delete(
                "",
            )


class TestAsyncUsers:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncIdentety) -> None:
        user = await async_client.users.create(
            address={
                "country": "USA",
                "locality": "New York",
                "postal_code": "10001",
                "region": "NY",
                "street_address": "123 Main St",
            },
            email="john@example.com",
            family_name="Doe",
            given_name="John",
            locale="en-US",
            metadata={"customField": "value"},
            name="John Doe",
            password="password123",
            picture="https://example.com/photo.jpg",
        )
        assert_matches_type(User, user, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncIdentety) -> None:
        response = await async_client.users.with_raw_response.create(
            address={
                "country": "USA",
                "locality": "New York",
                "postal_code": "10001",
                "region": "NY",
                "street_address": "123 Main St",
            },
            email="john@example.com",
            family_name="Doe",
            given_name="John",
            locale="en-US",
            metadata={"customField": "value"},
            name="John Doe",
            password="password123",
            picture="https://example.com/photo.jpg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = await response.parse()
        assert_matches_type(User, user, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncIdentety) -> None:
        async with async_client.users.with_streaming_response.create(
            address={
                "country": "USA",
                "locality": "New York",
                "postal_code": "10001",
                "region": "NY",
                "street_address": "123 Main St",
            },
            email="john@example.com",
            family_name="Doe",
            given_name="John",
            locale="en-US",
            metadata={"customField": "value"},
            name="John Doe",
            password="password123",
            picture="https://example.com/photo.jpg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = await response.parse()
            assert_matches_type(User, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncIdentety) -> None:
        user = await async_client.users.retrieve(
            "id",
        )
        assert_matches_type(User, user, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncIdentety) -> None:
        response = await async_client.users.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = await response.parse()
        assert_matches_type(User, user, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncIdentety) -> None:
        async with async_client.users.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = await response.parse()
            assert_matches_type(User, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncIdentety) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.users.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncIdentety) -> None:
        user = await async_client.users.update(
            id="id",
            address={
                "country": "USA",
                "locality": "New York",
                "postal_code": "10001",
                "region": "NY",
                "street_address": "123 Main St",
            },
            family_name="Doe",
            given_name="John",
            locale="en-US",
            metadata={"customField": "value"},
            name="John Doe",
            picture="https://example.com/photo.jpg",
        )
        assert_matches_type(User, user, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncIdentety) -> None:
        response = await async_client.users.with_raw_response.update(
            id="id",
            address={
                "country": "USA",
                "locality": "New York",
                "postal_code": "10001",
                "region": "NY",
                "street_address": "123 Main St",
            },
            family_name="Doe",
            given_name="John",
            locale="en-US",
            metadata={"customField": "value"},
            name="John Doe",
            picture="https://example.com/photo.jpg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = await response.parse()
        assert_matches_type(User, user, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncIdentety) -> None:
        async with async_client.users.with_streaming_response.update(
            id="id",
            address={
                "country": "USA",
                "locality": "New York",
                "postal_code": "10001",
                "region": "NY",
                "street_address": "123 Main St",
            },
            family_name="Doe",
            given_name="John",
            locale="en-US",
            metadata={"customField": "value"},
            name="John Doe",
            picture="https://example.com/photo.jpg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = await response.parse()
            assert_matches_type(User, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncIdentety) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.users.with_raw_response.update(
                id="",
                address={
                    "country": "USA",
                    "locality": "New York",
                    "postal_code": "10001",
                    "region": "NY",
                    "street_address": "123 Main St",
                },
                family_name="Doe",
                given_name="John",
                locale="en-US",
                metadata={"customField": "value"},
                name="John Doe",
                picture="https://example.com/photo.jpg",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncIdentety) -> None:
        user = await async_client.users.list(
            columns="id",
        )
        assert_matches_type(UserListResponse, user, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncIdentety) -> None:
        user = await async_client.users.list(
            columns="id",
            limit=0,
            page=0,
            sort="asc",
            sort_by="sortBy",
        )
        assert_matches_type(UserListResponse, user, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncIdentety) -> None:
        response = await async_client.users.with_raw_response.list(
            columns="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = await response.parse()
        assert_matches_type(UserListResponse, user, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncIdentety) -> None:
        async with async_client.users.with_streaming_response.list(
            columns="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = await response.parse()
            assert_matches_type(UserListResponse, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncIdentety) -> None:
        user = await async_client.users.delete(
            "id",
        )
        assert_matches_type(User, user, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncIdentety) -> None:
        response = await async_client.users.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = await response.parse()
        assert_matches_type(User, user, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncIdentety) -> None:
        async with async_client.users.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = await response.parse()
            assert_matches_type(User, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncIdentety) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.users.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .user import User as User
from .client import Client as Client
from .user_list_params import UserListParams as UserListParams
from .client_list_params import ClientListParams as ClientListParams
from .user_create_params import UserCreateParams as UserCreateParams
from .user_list_response import UserListResponse as UserListResponse
from .user_update_params import UserUpdateParams as UserUpdateParams
from .client_create_params import ClientCreateParams as ClientCreateParams
from .client_list_response import ClientListResponse as ClientListResponse
from .client_update_params import ClientUpdateParams as ClientUpdateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["UserListParams"]


class UserListParams(TypedDict, total=False):
    columns: Required[
        Literal[
            "id",
            "email",
            "name",
            "phone_number",
            "address",
            "locale",
            "email_verified",
            "phone_number_verified",
            "created_at",
            "updated_at",
        ]
    ]
    """Comma separated column names"""

    limit: float

    page: float

    sort: Literal["asc", "desc"]

    sort_by: Annotated[str, PropertyInfo(alias="sortBy")]
